@@ -1,75 +1,85 @@
-use std::{
-    convert::{TryFrom, TryInto},
-    fmt::Display,
-    sync::Mutex,
+use super::{AsymmetricCrypto, KeyPair};
+use crate::{
+    entropy::CsRng,
+    kdf::hkdf_256,
+    symmetric_crypto::{
+        aes_256_gcm_pure::{self, Aes256GcmCrypto},
+        nonce::NonceTrait,
+        SymmetricCrypto,
+    },
+    Error, KeyTrait,
 };
-
 use curve25519_dalek::{
     constants,
     ristretto::{CompressedRistretto, RistrettoPoint},
     scalar::Scalar,
 };
 use rand_core::{CryptoRng, RngCore};
-
-use super::{AsymmetricCrypto, KeyPair};
-use crate::{
-    entropy::CsRng,
-    kdf::hkdf_256,
-    symmetric_crypto::{aes_256_gcm_pure, Key, Nonce as _, SymmetricCrypto},
+use std::{
+    convert::{TryFrom, TryInto},
+    fmt::Display,
+    ops::{DerefMut, Mul},
+    sync::Mutex,
 };
 
 const HKDF_INFO: &[u8; 21] = b"ecies-ristretto-25519";
 
-/// For ECIES, cipher text needs to store:
-//
-// - the public key of the ephemeral keypair
-// - the AES nonce/iv
-// - the AES MAC
-pub const ECIES_ENCRYPTION_OVERHEAD: usize =
-    PUBLIC_KEY_LENGTH + aes_256_gcm_pure::NONCE_LENGTH + aes_256_gcm_pure::MAC_LENGTH;
-
-pub const PRIVATE_KEY_LENGTH: usize = 32;
-
 #[derive(Clone, PartialEq, Debug)]
 pub struct X25519PrivateKey(Scalar);
 
 impl X25519PrivateKey {
     #[must_use]
-    pub fn as_bytes(&self) -> [u8; 32] {
-        self.0.as_bytes().to_owned()
+    fn as_bytes(&self) -> &[u8] {
+        self.0.as_bytes()
+    }
+}
+
+impl KeyTrait for X25519PrivateKey {
+    const LENGTH: usize = 32;
+
+    fn new<R: RngCore + CryptoRng>(rng: &mut R) -> Self {
+        X25519PrivateKey(Scalar::random(rng))
     }
 
-    pub fn new<R: RngCore + CryptoRng>(rng: &mut R) -> Self {
-        X25519PrivateKey(Scalar::random(rng))
+    #[must_use]
+    fn to_bytes(&self) -> Vec<u8> {
+        self.as_bytes().to_vec()
+    }
+
+    fn parse(bytes: Vec<u8>) -> Result<Self, Error> {
+        Self::try_from(bytes)
+    }
+}
+
+impl TryFrom<Vec<u8>> for X25519PrivateKey {
+    type Error = Error;
+
+    fn try_from(bytes: Vec<u8>) -> Result<Self, Self::Error> {
+        Self::try_from(bytes.as_slice())
     }
 }
 
 impl TryFrom<&[u8]> for X25519PrivateKey {
-    type Error = anyhow::Error;
+    type Error = Error;
 
-    fn try_from(value: &[u8]) -> std::result::Result<Self, Self::Error> {
-        let len = value.len();
-        let bytes: [u8; PRIVATE_KEY_LENGTH] = value.try_into().map_err(|_e| {
-            anyhow::anyhow!(
-                "Invalid private key of length: {}, expected length: {}",
-                len,
-                PRIVATE_KEY_LENGTH
-            )
+    fn try_from(bytes: &[u8]) -> Result<Self, Self::Error> {
+        let len = bytes.len();
+        let bytes: [u8; <Self>::LENGTH] = bytes.try_into().map_err(|_| Error::SizeError {
+            given: len,
+            expected: <Self>::LENGTH,
         })?;
-        let scalar = Scalar::from_canonical_bytes(bytes)
-            .ok_or_else(|| anyhow::anyhow!("Invalid private key bytes"))?;
+        let scalar = Scalar::from_canonical_bytes(bytes).ok_or(Error::ConversionError)?;
         Ok(X25519PrivateKey(scalar))
     }
 }
 
 /// Parse from an hex encoded String
 impl TryFrom<&str> for X25519PrivateKey {
-    type Error = anyhow::Error;
+    type Error = Error;
 
     fn try_from(value: &str) -> std::result::Result<Self, Self::Error> {
-        let bytes = hex::decode(value)
-            .map_err(|e| anyhow::anyhow!("Invalid hex encoded private key: {}", e))?;
-        X25519PrivateKey::try_from(bytes.as_slice())
+        let bytes = hex::decode(value)?;
+        X25519PrivateKey::try_from(bytes)
     }
 }
 
@@ -80,15 +90,19 @@ impl Display for X25519PrivateKey {
     }
 }
 
-pub const PUBLIC_KEY_LENGTH: usize = 32; //compressed
-
 #[derive(Clone, PartialEq, Debug)]
 pub struct X25519PublicKey(RistrettoPoint);
 
-impl X25519PublicKey {
+impl KeyTrait for X25519PublicKey {
+    const LENGTH: usize = 32; //compressed
+
+    fn new<R: RngCore + CryptoRng>(rng: &mut R) -> Self {
+        Self::from(&X25519PrivateKey::new(rng))
+    }
+
     #[must_use]
-    pub fn as_bytes(&self) -> [u8; 32] {
-        self.0.compress().as_bytes().to_owned()
+    fn to_bytes(&self) -> Vec<u8> {
+        self.0.compress().to_bytes().to_vec()
     }
 }
 
@@ -98,33 +112,39 @@ impl From<&X25519PrivateKey> for X25519PublicKey {
     }
 }
 
-impl TryFrom<&[u8]> for X25519PublicKey {
-    type Error = anyhow::Error;
+impl TryFrom<Vec<u8>> for X25519PublicKey {
+    type Error = Error;
 
-    fn try_from(value: &[u8]) -> std::result::Result<Self, Self::Error> {
+    fn try_from(bytes: Vec<u8>) -> Result<Self, Self::Error> {
+        Self::try_from(bytes.as_slice())
+    }
+}
+
+impl TryFrom<&[u8]> for X25519PublicKey {
+    type Error = Error;
+
+    fn try_from(value: &[u8]) -> Result<Self, Self::Error> {
         let len = value.len();
-        if len != PUBLIC_KEY_LENGTH {
-            anyhow::bail!(
-                "Invalid key of length: {}, expected length: {}",
-                len,
-                PUBLIC_KEY_LENGTH
-            )
+        if len != <X25519PublicKey>::LENGTH {
+            return Err(Error::SizeError {
+                given: len,
+                expected: <X25519PublicKey>::LENGTH,
+            });
         };
         let compressed = CompressedRistretto::from_slice(value);
         let point = compressed
             .decompress()
-            .ok_or_else(|| anyhow::anyhow!("Could nos decompress the Ristretto point"))?;
+            .ok_or(Self::Error::ConversionError)?;
         Ok(X25519PublicKey(point))
     }
 }
 
 /// Parse from an hex encoded String
 impl TryFrom<&str> for X25519PublicKey {
-    type Error = anyhow::Error;
+    type Error = Error;
 
     fn try_from(value: &str) -> std::result::Result<Self, Self::Error> {
-        let bytes = hex::decode(value)
-            .map_err(|e| anyhow::anyhow!("Invalid hex encoded public key: {}", e))?;
+        let bytes = hex::decode(value)?;
         X25519PublicKey::try_from(bytes.as_slice())
     }
 }
@@ -136,28 +156,41 @@ impl Display for X25519PublicKey {
     }
 }
 
-#[derive(Clone, PartialEq, Debug)]
-pub struct X25519KeyPair {
-    pub private_key: X25519PrivateKey,
-    pub public_key: X25519PublicKey,
+impl<'a, 'b> Mul<&'a X25519PrivateKey> for &'b X25519PublicKey {
+    type Output = X25519PublicKey;
+
+    fn mul(self, rhs: &'a X25519PrivateKey) -> Self::Output {
+        X25519PublicKey(self.0 * rhs.0)
+    }
 }
 
-impl X25519KeyPair {
-    pub fn new<R: RngCore + CryptoRng>(rng: &mut R) -> X25519KeyPair {
-        let scalar = Scalar::random(rng);
-        let public_key = X25519PublicKey(&scalar * &constants::RISTRETTO_BASEPOINT_TABLE);
-        let private_key = X25519PrivateKey(scalar);
-        X25519KeyPair {
-            private_key,
-            public_key,
-        }
+impl<'a> Mul<&'a X25519PrivateKey> for X25519PublicKey {
+    type Output = X25519PublicKey;
+
+    fn mul(self, rhs: &'a X25519PrivateKey) -> Self::Output {
+        X25519PublicKey(self.0 * rhs.0)
     }
 }
 
+#[derive(Clone, PartialEq, Debug)]
+pub struct X25519KeyPair {
+    private_key: X25519PrivateKey,
+    public_key: X25519PublicKey,
+}
+
 impl KeyPair for X25519KeyPair {
     type PrivateKey = X25519PrivateKey;
     type PublicKey = X25519PublicKey;
 
+    fn new<R: RngCore + CryptoRng>(rng: &mut R) -> X25519KeyPair {
+        let private_key = X25519PrivateKey::new(rng);
+        let public_key = X25519PublicKey::from(&private_key);
+        X25519KeyPair {
+            private_key,
+            public_key,
+        }
+    }
+
     fn public_key(&self) -> &Self::PublicKey {
         &self.public_key
     }
@@ -167,20 +200,40 @@ impl KeyPair for X25519KeyPair {
     }
 }
 
-impl TryFrom<&[u8]> for X25519KeyPair {
-    type Error = anyhow::Error;
+#[allow(clippy::from_over_into)]
+impl Into<Vec<u8>> for X25519KeyPair {
+    fn into(self) -> Vec<u8> {
+        let mut bytes = self.private_key().to_bytes();
+        bytes.append(&mut self.public_key().to_bytes());
+        bytes
+    }
+}
 
-    fn try_from(value: &[u8]) -> Result<Self, Self::Error> {
-        let len = value.len();
-        if len != PRIVATE_KEY_LENGTH + PUBLIC_KEY_LENGTH {
-            anyhow::bail!(
-                "Invalid key pair of length: {}, expected length: {}",
-                len,
-                PRIVATE_KEY_LENGTH + PUBLIC_KEY_LENGTH
-            );
+impl TryFrom<Vec<u8>> for X25519KeyPair {
+    type Error = Error;
+
+    fn try_from(bytes: Vec<u8>) -> Result<Self, Self::Error> {
+        Self::try_from(bytes.as_slice())
+    }
+}
+
+impl TryFrom<&[u8]> for X25519KeyPair {
+    type Error = Error;
+
+    fn try_from(bytes: &[u8]) -> Result<Self, Self::Error> {
+        let len = <Self as KeyPair>::PrivateKey::LENGTH + <Self as KeyPair>::PublicKey::LENGTH;
+        if len != bytes.len() {
+            return Err(Error::SizeError {
+                given: bytes.len(),
+                expected: len,
+            });
         }
-        let private_key = X25519PrivateKey::try_from(&value[0..PRIVATE_KEY_LENGTH])?;
-        let public_key = X25519PublicKey::try_from(&value[PRIVATE_KEY_LENGTH..])?;
+        let private_key = <Self as KeyPair>::PrivateKey::try_from(
+            &bytes[..<Self as KeyPair>::PrivateKey::LENGTH],
+        )?;
+        let public_key = <Self as KeyPair>::PublicKey::try_from(
+            &bytes[<Self as KeyPair>::PrivateKey::LENGTH..],
+        )?;
         Ok(X25519KeyPair {
             private_key,
             public_key,
@@ -190,12 +243,12 @@ impl TryFrom<&[u8]> for X25519KeyPair {
 
 impl Display for X25519KeyPair {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}{}", &self.private_key, &self.public_key)
+        write!(f, "{}{}", self.private_key(), self.public_key())
     }
 }
 
 pub struct X25519Crypto {
-    rng: Mutex<CsRng>,
+    pub(crate) rng: Mutex<CsRng>,
 }
 
 impl Default for X25519Crypto {
@@ -217,6 +270,14 @@ impl PartialEq for X25519Crypto {
 }
 
 impl X25519Crypto {
+    /// For ECIES, cipher text needs to store:
+    //
+    // - the public key of the ephemeral keypair
+    // - the AES nonce/iv
+    // - the AES MAC
+    pub const ENCRYPTION_OVERHEAD: usize =
+        <X25519PublicKey>::LENGTH + aes_256_gcm_pure::NONCE_LENGTH + aes_256_gcm_pure::MAC_LENGTH;
+
     /// Generate a 256 bit symmetric key used with ECIES encryption
     pub fn sym_key_from_public_key(
         ephemeral_keypair: &X25519KeyPair, // (y, gʸ)
@@ -225,11 +286,14 @@ impl X25519Crypto {
         //calculate the shared point: (gˣ)ʸ
         let point = public_key.0 * ephemeral_keypair.private_key.0;
         // create a 64 bytes master key using gʸ and the shared point
-        let mut master = [0_u8; 2 * PUBLIC_KEY_LENGTH];
-        master[..PUBLIC_KEY_LENGTH].clone_from_slice(&ephemeral_keypair.public_key.as_bytes());
-        master[PUBLIC_KEY_LENGTH..].clone_from_slice(&point.compress().to_bytes());
+        let mut master = [0_u8; 2 * <X25519PublicKey>::LENGTH];
+        master[..<X25519PublicKey>::LENGTH]
+            .clone_from_slice(&ephemeral_keypair.public_key.to_bytes());
+        master[<X25519PublicKey>::LENGTH..].clone_from_slice(&point.compress().to_bytes());
         //Derive a 256 bit key using HKDF
-        hkdf_256(&master, HKDF_INFO)
+        Ok(hkdf_256(&master, 32, HKDF_INFO)?
+            .try_into()
+            .expect("Size should be okay"))
     }
 
     /// Generate a 256 bit symmetric key used with ECIES decryption
@@ -240,21 +304,13 @@ impl X25519Crypto {
         //calculate the shared point: (gʸ)ˣ
         let point = private_key.0 * ephemeral_public_key.0;
         // create a 64 bytes master key using gʸ and the shared point
-        let mut master = [0_u8; 2 * PUBLIC_KEY_LENGTH];
-        master[..PUBLIC_KEY_LENGTH].clone_from_slice(&ephemeral_public_key.as_bytes());
-        master[PUBLIC_KEY_LENGTH..].clone_from_slice(&point.compress().to_bytes());
+        let mut master = [0_u8; 2 * <X25519PublicKey>::LENGTH];
+        master[..<X25519PublicKey>::LENGTH].clone_from_slice(&ephemeral_public_key.to_bytes());
+        master[<X25519PublicKey>::LENGTH..].clone_from_slice(&point.compress().to_bytes());
         //Derive a 256 bit key using HKDF
-        hkdf_256(&master, HKDF_INFO)
-    }
-
-    pub fn new_private_key(&self) -> X25519PrivateKey {
-        let rng = &mut *self.rng.lock().expect("a mutex lock failed");
-        X25519PrivateKey::new(rng)
-    }
-
-    pub fn fill_bytes(&self, dest: &mut [u8]) {
-        let rng = &mut *self.rng.lock().expect("a mutex lock failed");
-        rng.fill_bytes(dest);
+        Ok(hkdf_256(&master, 32, HKDF_INFO)?
+            .try_into()
+            .expect("Size should be okay"))
     }
 }
 
@@ -282,8 +338,9 @@ impl AsymmetricCrypto for X25519Crypto {
         &self,
         _: Option<&Self::PrivateKeyGenerationParameters>,
     ) -> anyhow::Result<<Self::KeyPair as KeyPair>::PrivateKey> {
-        let rng = &mut *self.rng.lock().expect("a mutex lock failed");
-        Ok(X25519PrivateKey::new(rng))
+        Ok(X25519PrivateKey::new(
+            &mut self.rng.lock().expect("a lock failed").deref_mut(),
+        ))
     }
 
     /// Generate a key pair, private key and public key
@@ -291,8 +348,9 @@ impl AsymmetricCrypto for X25519Crypto {
         &self,
         _: Option<&Self::KeyPairGenerationParameters>,
     ) -> anyhow::Result<Self::KeyPair> {
-        let rng = &mut *self.rng.lock().expect("a mutex lock failed");
-        Ok(X25519KeyPair::new(rng))
+        Ok(X25519KeyPair::new(
+            &mut self.rng.lock().expect("a lock failed").deref_mut(),
+        ))
     }
 
     /// Generate a symmetric key, and its encryption,to be used in an hybrid
@@ -303,8 +361,8 @@ impl AsymmetricCrypto for X25519Crypto {
         _: Option<&Self::EncryptionParameters>,
     ) -> anyhow::Result<(S::Key, Vec<u8>)> {
         let bytes: Vec<u8> = self.generate_random_bytes(S::Key::LENGTH);
-        let symmetric_key = S::generate_key_from_rnd(&bytes)?;
-        let encrypted_key = self.encrypt(public_key, None, &symmetric_key.as_bytes())?;
+        let symmetric_key = S::Key::try_from(bytes)?;
+        let encrypted_key = self.encrypt(public_key, None, &symmetric_key.to_bytes())?;
         Ok((symmetric_key, encrypted_key))
     }
 
@@ -313,8 +371,11 @@ impl AsymmetricCrypto for X25519Crypto {
         &self,
         private_key: &<Self::KeyPair as KeyPair>::PrivateKey,
         data: &[u8],
-    ) -> anyhow::Result<S::Key> {
-        S::Key::parse(self.decrypt(private_key, data)?)
+    ) -> Result<S::Key, Error> {
+        S::Key::parse(
+            self.decrypt(private_key, data)
+                .map_err(|err| Error::DecryptionError(err.to_string()))?,
+        )
     }
 
     /// A utility function to generate random bytes from an uniform distribution
@@ -328,7 +389,7 @@ impl AsymmetricCrypto for X25519Crypto {
 
     /// The encrypted message length
     fn encrypted_message_length(&self, clear_text_message_length: usize) -> usize {
-        clear_text_message_length + ECIES_ENCRYPTION_OVERHEAD
+        clear_text_message_length + <Self>::ENCRYPTION_OVERHEAD
     }
 
     /// Encrypt a message using ECIES
@@ -339,27 +400,27 @@ impl AsymmetricCrypto for X25519Crypto {
         _: Option<&Self::EncryptionParameters>,
         data: &[u8],
     ) -> anyhow::Result<Vec<u8>> {
-        let rng = &mut *self.rng.lock().expect("a mutex lock failed");
-        let ephemeral_keypair = X25519KeyPair::new(rng);
+        let ephemeral_keypair =
+            X25519KeyPair::new(&mut self.rng.lock().expect("a lock failed").deref_mut());
         let sym_key_bytes = Self::sym_key_from_public_key(&ephemeral_keypair, public_key)?;
         // use the pure rust aes implementation
-        let sym_key = aes_256_gcm_pure::Key(sym_key_bytes);
-        let aes = aes_256_gcm_pure::Aes256GcmCrypto::new();
-        let nonce = aes.generate_nonce();
+        let sym_key = aes_256_gcm_pure::Key::from(sym_key_bytes);
+        let nonce =
+            aes_256_gcm_pure::Nonce::new(&mut self.rng.lock().expect("a lock failed").deref_mut());
         //prepare the result
-        let mut result: Vec<u8> = Vec::with_capacity(data.len() + ECIES_ENCRYPTION_OVERHEAD);
-        result.extend_from_slice(&ephemeral_keypair.public_key.as_bytes());
+        let mut result: Vec<u8> = Vec::with_capacity(data.len() + <Self>::ENCRYPTION_OVERHEAD);
+        result.extend_from_slice(&ephemeral_keypair.public_key.to_bytes());
         result.extend_from_slice(&nonce.0);
-        result.extend(aes.encrypt(&sym_key, data, &nonce, None)?);
+        result.extend(Aes256GcmCrypto::encrypt(&sym_key, data, &nonce, None)?);
         Ok(result)
     }
 
     /// The decrypted message length
     fn clear_text_message_length(encrypted_message_length: usize) -> usize {
-        if encrypted_message_length <= ECIES_ENCRYPTION_OVERHEAD {
+        if encrypted_message_length <= <Self>::ENCRYPTION_OVERHEAD {
             0
         } else {
-            encrypted_message_length - ECIES_ENCRYPTION_OVERHEAD
+            encrypted_message_length - <Self>::ENCRYPTION_OVERHEAD
         }
     }
 
@@ -370,25 +431,24 @@ impl AsymmetricCrypto for X25519Crypto {
         private_key: &<Self::KeyPair as KeyPair>::PrivateKey,
         data: &[u8],
     ) -> anyhow::Result<Vec<u8>> {
-        if data.len() < ECIES_ENCRYPTION_OVERHEAD {
+        if data.len() < <Self>::ENCRYPTION_OVERHEAD {
             anyhow::bail!("decryption failed: message is too short");
         }
-        if data.len() == ECIES_ENCRYPTION_OVERHEAD {
+        if data.len() == <Self>::ENCRYPTION_OVERHEAD {
             return Ok(vec![]);
         }
         // gʸ
-        let ephemeral_public_key_bytes = &data[0..PUBLIC_KEY_LENGTH];
+        let ephemeral_public_key_bytes = &data[0..<X25519PublicKey>::LENGTH];
         let ephemeral_public_key = X25519PublicKey::try_from(ephemeral_public_key_bytes)?;
         let sym_key_bytes = Self::sym_key_from_private_key(&ephemeral_public_key, private_key)?;
         // use the pure rust aes implementation
-        let sym_key = aes_256_gcm_pure::Key(sym_key_bytes);
-        let aes = aes_256_gcm_pure::Aes256GcmCrypto::new();
-        let nonce_bytes =
-            &data[PUBLIC_KEY_LENGTH..PUBLIC_KEY_LENGTH + aes_256_gcm_pure::NONCE_LENGTH];
+        let sym_key = aes_256_gcm_pure::Key::from(sym_key_bytes);
+        let nonce_bytes = &data
+            [<X25519PublicKey>::LENGTH..<X25519PublicKey>::LENGTH + aes_256_gcm_pure::NONCE_LENGTH];
         let nonce = aes_256_gcm_pure::Nonce::try_from_slice(nonce_bytes)?;
-        aes.decrypt(
+        Aes256GcmCrypto::decrypt(
             &sym_key,
-            &data[PUBLIC_KEY_LENGTH + aes_256_gcm_pure::NONCE_LENGTH..],
+            &data[<X25519PublicKey>::LENGTH + aes_256_gcm_pure::NONCE_LENGTH..],
             &nonce,
             None,
         )
@@ -399,30 +459,28 @@ impl AsymmetricCrypto for X25519Crypto {
 mod test {
     use std::convert::TryFrom;
 
-    use super::{AsymmetricCrypto, KeyPair};
-    use crate::{
-        asymmetric::ristretto::ECIES_ENCRYPTION_OVERHEAD, symmetric_crypto::aes_256_gcm_pure,
-    };
+    use super::{AsymmetricCrypto, KeyPair, X25519Crypto, X25519PrivateKey, X25519PublicKey};
+    use crate::{symmetric_crypto::aes_256_gcm_pure, KeyTrait};
 
     #[test]
     fn test_generate_key_pair() {
         let crypto = super::X25519Crypto::new();
         let key_pair_1 = crypto.generate_key_pair(None).unwrap();
         assert_ne!(
-            &[0_u8; super::PRIVATE_KEY_LENGTH],
+            &[0_u8; X25519PrivateKey::LENGTH],
             key_pair_1.private_key.0.as_bytes()
         );
         assert_ne!(
-            [0_u8; super::PUBLIC_KEY_LENGTH],
-            key_pair_1.public_key.as_bytes()
+            vec![0_u8; X25519PublicKey::LENGTH],
+            key_pair_1.public_key.to_bytes()
         );
         assert_eq!(
-            super::PRIVATE_KEY_LENGTH as usize,
+            X25519PrivateKey::LENGTH as usize,
             key_pair_1.private_key.0.as_bytes().len()
         );
         assert_eq!(
-            super::PUBLIC_KEY_LENGTH as usize,
-            key_pair_1.public_key.as_bytes().len()
+            X25519PublicKey::LENGTH as usize,
+            key_pair_1.public_key.to_bytes().len()
         );
         let key_pair_2 = crypto.generate_key_pair(None).unwrap();
         assert_ne!(key_pair_2.private_key, key_pair_1.private_key);
@@ -459,7 +517,7 @@ mod test {
             .encrypt(&key_pair.public_key, None, &random_msg)
             .unwrap();
         assert_eq!(
-            4096_usize + ECIES_ENCRYPTION_OVERHEAD,
+            4096_usize + <X25519Crypto>::ENCRYPTION_OVERHEAD,
             crypto.encrypted_message_length(random_msg.len())
         );
         assert_eq!(
