@@ -1,147 +1,22 @@
-use std::{
-    cmp::min,
-    collections::HashMap,
-    convert::{TryFrom, TryInto},
-    fmt::Display,
-    str::FromStr,
-    sync::Mutex,
-    vec::Vec,
-};
-
+use crate::symmetric_crypto::SymmetricCrypto;
 use aes::Aes256;
 use cosmian_fpe::ff1::{FlexibleNumeralString, FF1};
 use itertools::Itertools;
 use num_traits::Bounded;
-use rand::{RngCore, SeedableRng};
-use rand_hc::Hc128Rng;
+use std::{collections::HashMap, convert::TryFrom, fmt::Display, str::FromStr, vec::Vec};
 use tracing::trace;
 
-use super::SymmetricCrypto;
-use crate::symmetric_crypto::Key as _;
-
 pub const RECOMMENDED_THRESHOLD: usize = 1_000_000;
 pub const KEY_LENGTH: usize = 32;
 pub const NONCE_LENGTH: usize = 0;
 pub const MAC_LENGTH: usize = 0;
 
-#[derive(Debug, Clone, PartialEq)]
-pub struct Key(pub [u8; KEY_LENGTH]);
+pub type Key = crate::symmetric_crypto::key::Key<KEY_LENGTH>;
+pub type Nonce = crate::symmetric_crypto::nonce::Nonce<NONCE_LENGTH>;
 
-impl super::Key for Key {
-    const LENGTH: usize = KEY_LENGTH;
-
-    fn try_from(bytes: Vec<u8>) -> anyhow::Result<Self> {
-        Self::try_from_slice(bytes.as_slice())
-    }
-
-    fn try_from_slice(bytes: &[u8]) -> anyhow::Result<Self> {
-        let len = bytes.len();
-        let b: [u8; KEY_LENGTH] = bytes.try_into().map_err(|_| {
-            anyhow::anyhow!(
-                "Invalid key of length: {}, expected length: {}",
-                len,
-                KEY_LENGTH
-            )
-        })?;
-        Ok(Self(b))
-    }
-
-    fn as_bytes(&self) -> Vec<u8> {
-        self.0.to_vec()
-    }
-}
-
-impl Key {
-    #[must_use]
-    pub fn as_array(&self) -> [u8; 32] {
-        self.0
-    }
-}
-
-impl From<Key> for Vec<u8> {
-    fn from(k: Key) -> Vec<u8> {
-        k.0.to_vec()
-    }
-}
-
-impl Display for Key {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}", hex::encode(self.0))
-    }
-}
-
-#[derive(Clone, Debug, PartialEq)]
-pub struct Nonce(pub [u8; NONCE_LENGTH]);
-
-impl super::Nonce for Nonce {
-    const LENGTH: usize = NONCE_LENGTH;
-
-    fn try_from(bytes: Vec<u8>) -> anyhow::Result<Self> {
-        Self::try_from_slice(bytes.as_slice())
-    }
-
-    fn try_from_slice(bytes: &[u8]) -> anyhow::Result<Self> {
-        let len = bytes.len();
-        let b: [u8; NONCE_LENGTH] = bytes.try_into().map_err(|_| {
-            anyhow::anyhow!(
-                "Invalid nonce of length: {}, expected length: {}",
-                len,
-                NONCE_LENGTH
-            )
-        })?;
-        Ok(Self(b))
-    }
-
-    fn increment(&self, increment: usize) -> Self {
-        let mut vec = self.0.to_vec();
-        vec.extend_from_slice(&[0_u8; 128 - Self::LENGTH]);
-        let mut v = u128::from_le_bytes(
-            vec.try_into()
-                .expect("This should never happen: nonce is 96 bit < 128 bits"),
-        );
-        v += increment as u128;
-        Nonce(
-            v.to_be_bytes()[0..Self::LENGTH]
-                .try_into()
-                .expect("This should never happen: nonce is 96 bit < 128 bits"),
-        )
-    }
-
-    fn xor(&self, b2: &[u8]) -> Self {
-        let mut n = self.0;
-        for i in 0..min(b2.len(), NONCE_LENGTH) {
-            n[i] ^= b2[i];
-        }
-        Nonce(n)
-    }
-
-    fn as_bytes(&self) -> Vec<u8> {
-        self.0.to_vec()
-    }
-}
-
-impl From<Nonce> for Vec<u8> {
-    fn from(n: Nonce) -> Vec<u8> {
-        n.0.to_vec()
-    }
-}
-
-impl From<Key> for [u8; KEY_LENGTH] {
-    fn from(k: Key) -> [u8; KEY_LENGTH] {
-        k.0
-    }
-}
-
-impl Display for Nonce {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}", hex::encode(self.0))
-    }
-}
 #[allow(clippy::upper_case_acronyms)]
 #[derive(Default)]
-pub struct FF1Crypto {
-    rng: Mutex<CsRng>,
-}
+pub struct FF1Crypto;
 
 impl Display for FF1Crypto {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -378,7 +253,7 @@ impl FF1Crypto {
     ) -> anyhow::Result<Vec<u8>> {
         let plaintext = plaintext.into_iter().map(u16::from).collect::<Vec<_>>();
         let ciphertext = Self::encrypt_u16(key, tweak, radix, plaintext)?;
-        let mut result = Vec::<u8>::new();
+        let mut result = Vec::with_capacity(ciphertext.len());
         for e in ciphertext {
             result.push(u8::try_from(e)?);
         }
@@ -393,7 +268,7 @@ impl FF1Crypto {
     ) -> anyhow::Result<Vec<u8>> {
         let ciphertext = ciphertext.into_iter().map(u16::from).collect::<Vec<_>>();
         let cleartext = Self::decrypt_u16(key, tweak, radix, ciphertext)?;
-        let mut result = Vec::<u8>::new();
+        let mut result = Vec::with_capacity(cleartext.len());
         for e in cleartext {
             result.push(u8::try_from(e)?);
         }
@@ -551,41 +426,11 @@ impl SymmetricCrypto for FF1Crypto {
 
     const MAC_LENGTH: usize = MAC_LENGTH;
 
-    #[must_use]
-    fn new() -> Self {
-        FF1Crypto {
-            rng: Mutex::new(CsRng::new()),
-        }
-    }
-
     fn description() -> String {
         format!("FF1 pure Rust (key bits: {})", KEY_LENGTH * 8,)
     }
 
-    fn generate_random_bytes(&self, len: usize) -> Vec<u8> {
-        self.rng
-            .lock()
-            .expect("a mutex lock failed")
-            .generate_random_bytes(len)
-    }
-
-    fn generate_key_from_rnd(rnd_bytes: &[u8]) -> anyhow::Result<Self::Key> {
-        Self::Key::try_from_slice(rnd_bytes)
-    }
-
-    fn generate_key(&self) -> Self::Key {
-        self.rng.lock().expect("a mutex lock failed").generate_key()
-    }
-
-    fn generate_nonce(&self) -> Self::Nonce {
-        self.rng
-            .lock()
-            .expect("a mutex lock failed")
-            .generate_nonce()
-    }
-
     fn encrypt(
-        &self,
         key: &Self::Key,
         bytes: &[u8],
         _nonce: &Self::Nonce,
@@ -595,7 +440,6 @@ impl SymmetricCrypto for FF1Crypto {
     }
 
     fn decrypt(
-        &self,
         key: &Self::Key,
         bytes: &[u8],
         _nonce: &Self::Nonce,
@@ -605,55 +449,6 @@ impl SymmetricCrypto for FF1Crypto {
     }
 }
 
-/// A cryptographically secure RNG for use with AES 256
-/// Using this struct avoids having to
-/// gather entropy every time which is slow when
-/// generating Nonces
-pub struct CsRng {
-    rng: Hc128Rng,
-}
-
-impl CsRng {
-    #[must_use]
-    pub fn new() -> Self {
-        Self {
-            rng: Hc128Rng::from_entropy(),
-        }
-    }
-
-    /// Generate an vector of random bytes
-    pub fn generate_random_bytes(&mut self, len: usize) -> Vec<u8> {
-        let mut bytes = vec![0_u8; len];
-        self.rng.fill_bytes(&mut bytes);
-        bytes
-    }
-
-    /// Fill `dest ` with random bytes
-    pub fn fill_bytes(&mut self, dest: &mut [u8]) {
-        self.rng.fill_bytes(dest);
-    }
-
-    /// Generate a fresh nonce
-    pub fn generate_nonce(&mut self) -> Nonce {
-        let mut nonce = Nonce([0_u8; NONCE_LENGTH]);
-        self.rng.fill_bytes(&mut nonce.0);
-        nonce
-    }
-
-    /// Generate a new symmetric random `Key`
-    pub fn generate_key(&mut self) -> Key {
-        let mut key = Key([0_u8; KEY_LENGTH]);
-        self.rng.fill_bytes(&mut key.0);
-        key
-    }
-}
-
-impl Default for CsRng {
-    fn default() -> Self {
-        CsRng::new()
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use rand::{thread_rng, Rng};
